/*

	INSTALL.C -- Install EGAD Screen Print from floppy

	EGAD

	Copyright (c) 1993 LS Software

	8139 E. Mawson
	Mesa, AZ 85207
	(602) 380-9175

	All Rights Reserved. No part of this program may be reproduced
	in whole or in part without express permission of LS Software.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <memory.h>
#include <string.h>
#include <dir.h>
#include <ctype.h>
#include "keys.h"
#include "screen.h"
#include "version.h"

/* --- Should change this array to a malloc'ed structure --- */

#define MAXPRINTERS 150

struct printer {
   char printer_name[36];
   int driver_type;
   struct printer *next;
} printertype[MAXPRINTERS];
char *cpPrinterName[MAXPRINTERS];
int nprinters=0;

char driver_name[40][14];
char driver_key[40][36];
int ndrivers=0;

char egad_name[]="EGAD Screen Print";

static char *sSystemFile[]={"AUTOEXEC.BAT","CONFIG.SYS"};
static char sCompatFile[]="_PRINTER.DAT";
static char sExeExtension[]=".EXE";
static char sEgad[]="EGAD";

/* Place to build menus, instruction boxes, etc. */
static char msTemp[500];

/* Read the printer compatibility file.
 * format is (example):
 * --------------------
 *	*egadep.exe
 *	=Epson & Compatibles 9-pin
 *	Epson FX Series
 *	Epson LX-800
 *	
 * --------------------
 */

int read_compat(char *sFromDisk)
   {
      char sCompatPath[65];
      FILE *pCompatPath;
      char *p;
      int nInsert;
      struct printer *pInsert;

      strcpy(sCompatPath,sFromDisk);
      strcat(sCompatPath,sCompatFile);
      pCompatPath = fopen(sCompatPath,"r");
      if (pCompatPath == NULL) {
	 return(0);
      }
      strcpy(printertype[0].printer_name,"~");
      printertype[0].next=NULL;

      while (!feof(pCompatPath)) {
	 if (fgets(msTemp,80,pCompatPath) != NULL) {
	    p = &msTemp[strlen(msTemp)-1];
	    if (*p == '\n') *p = '\0';		/* remove trailing \n */
	    p = &msTemp[0];
	    switch (msTemp[0]) {
	       case '*':
		  strupr(p);
		  strcpy(driver_name[ndrivers],++p);
		  ndrivers++;
		  break;
	       case '=':
		  ++p;
		  if (*p == '=') {
		     strcpy(driver_key[ndrivers-1],++p);
		     /* Fall through */
		  } else {
		     strcpy(driver_key[ndrivers-1],p);
		     break;
		  }
	       default:
		  if (msTemp[0] <= '@') break;	/* Ignore comments &c. */
		  if (strlen(p) && (nprinters < (MAXPRINTERS-1))) {
		     nprinters++;
		     nInsert=0;
		     pInsert=&printertype[nInsert];
		     while ((pInsert->next != NULL) &&
			   (stricmp(pInsert->next -> printer_name, p) < 0)) {
			pInsert=pInsert->next;
		     }
		     printertype[nprinters].next=pInsert->next;
		     pInsert->next = &printertype[nprinters];
		     strcpy(printertype[nprinters].printer_name,p);
		     printertype[nprinters].driver_type=ndrivers-1;
		  }
		  break;
	    }
	 }
      }
      fclose(pCompatPath);
      nInsert=0;
      pInsert=printertype[0].next;
      while (pInsert != NULL) {
	 cpPrinterName[nInsert++]=pInsert->printer_name;
	 pInsert = pInsert -> next;
      }
      return(1);
   }

#define DETECT_EGAD 1
#define DETECT_EGAD_CONFIG 2
#define DETECT_MENU_SYSTEM 4

int iDetectSystem(char *sBootDisk)
   /* Returns bit field:
    *    - - - - - M F E
    *                  +-- 1 = EGAD found
    *		     +---- 0 =   Found in CONFIG.SYS
    *			   1 =   Found in AUTOEXEC.BAT
    *		   +------ 1 = Menu system (Windows, DOSSHELL) found
    */
   {
      char *p, *q;
      FILE *fpInfile;
      int nSysFile;
      static char *sMenuSystem[]={"WIN","DOSSHELL"};
      int iEgadFound=0;
      int bMenuFound=0;
      int nPgmFound;

      for (nSysFile=0;nSysFile<=1;nSysFile++) {
	 strcpy(msTemp,sBootDisk);
	 strcat(msTemp,"\\");
	 strcat(msTemp,sSystemFile[nSysFile]);
	 fpInfile = fopen(msTemp,"r");
	 if (fpInfile != NULL) {
	    while (!feof(fpInfile) && (fgets(msTemp,132,fpInfile) != NULL)) {
	       strupr(msTemp);
	       p = msTemp;
	       /* Skip over leading spaces & blanks */
	       while ((*p == 32) || (*p == 9)) p++;
	       /* Skip lines starting with "REM" */
	       if ((*p=='R') && (*(p+1)=='E') && (*(p+2)=='M')) continue;
	       /* Command must include "/T" to be EGAD/T: */
	       if (strstr(p,"/T") == NULL) continue;
	       /* Now look for "EGAD" followed by a non-alphanumeric character */
	       nPgmFound = 0;		/* No program recognized */
	       q = strstr(p,sEgad);
	       if (q != NULL) {
		  nPgmFound = 1;	/* EGAD! */
	       } else {
		  if ((q = strstr(p,sMenuSystem[0])) != NULL) {
		     nPgmFound = 2;	/* WIN */
		  } else {
		     if ((q = strstr(p,sMenuSystem[1])) != NULL) {
			nPgmFound = 3;	/* DOSSHELL */
		     }
		  }
	       }
	       if (q == NULL) {
		  continue;		/* none found */
	       }
	       if (isalnum(*(q+strlen(sEgad)))) {
		  continue;		/* e.g., "EGADS" */
	       }
	       /* If not found at beginning of string (q==p) check before too */
	       if ((q != p) && isalnum(*(q-1))) {
		  continue;		/* e.g., "BEGAD" */
	       }
	       if (nPgmFound > 1) {
		  bMenuFound = 1;
	       } else {
		  iEgadFound = (nSysFile<<1)+DETECT_EGAD;
	       }
	    }
	    fclose(fpInfile);
	 }
      }
      return((bMenuFound << 2)+(iEgadFound));
   }

void ShowFileError(char *message, char *fname)
   {
      strcpy(msTemp,message);
      strcat(msTemp,fname);
      strcat(msTemp,":\n   ");
      strcat(msTemp,_strerror(NULL));
#if 0
      strcat(msTemp,"\n");
#endif
#if 1
      show_error(msTemp,10,15,1);
#else
      strcat(msTemp,"\nPress <Esc> to exit");
      instruction(msTemp,10,15);
      while (gtkey() != 27) ;
      erase_instruction();
#endif
   }

int iModifySystem(char *sEgadPath, char *sBootDisk, int nSysFile)
   {
      FILE *fpOutFile;
      char sSysFilePath[64];

      strcpy(sSysFilePath,sBootDisk);
      strcat(sSysFilePath,"\\");
      strcat(sSysFilePath,sSystemFile[nSysFile]);
      if ((fpOutFile = fopen(sSysFilePath,"a")) == NULL) {
	 ShowFileError("Error writing ",sSysFilePath);
	 return(0);
      }
      switch (nSysFile) {
	 case 0:	/* AUTOEXEC.BAT */
	    fputs("\n",fpOutFile);
	    break;
	 case 1:	/* CONFIG.SYS */
	    fputs("\nINSTALL=",fpOutFile);
	    break;
      }
      fputs(sEgadPath,fpOutFile);
      fputs(" /T\n",fpOutFile);
      fclose(fpOutFile);
      return(1);
   }

int nFindBestSysMod()
   /* Returns
    *  -1  cannot modify (DOS 1 or 2)
    *   0  modify AUTOEXEC.BAT
    *   1  modify CONFIG.SYS
    */
   {
      switch (_osmajor) {
	 case 1:
	 case 2:
	    return(-1);
	 case 3:
	 case 4:
	    return(0);	/* install into AUTOEXEC.BAT */
	 default:
	    return(1);	/* install into CONFIG.SYS */
      }
   }

char *sAskWhichBoots()
   {
      static char *mnuBootDisks[]={"C:","A:"};
      static char sInstruction[]=
"Select the disk you want\n\
INSTALL to modify, so that\n\
EGAD will load each time you\n\
start your computer. Most\n\
users with hard disks should\n\
select C:.\n";
      int choice;

      instruction(sInstruction,11,5);
      choice = menu("Which drive you boot from?",mnuBootDisks,2,10,40);
      erase_instruction();
      if (choice >= 0) {
	 return(mnuBootDisks[choice]);
      } else {
	 return(NULL);
      }
   }

/* returns TRUE or FALSE */
int bAskModifySpecific(char *sBootDisk, int nSysFile)
   {
      static char *mnuYesNo[]={"~Yes","~No -- exit"};
      int choice;

      strcpy(msTemp,"INSTALL will now modify\n");
      strcat(msTemp,sBootDisk);
      strcat(msTemp,"\\");
      strcat(msTemp,sSystemFile[nSysFile]);
      strcat(msTemp,"\n");
      instruction(msTemp,10,10);
      choice = menu("Proceed?",mnuYesNo,2,10,40);
      erase_instruction();
      return(choice == 0);
   }

void AskIfModify(char *sEgadPath)
   {
      static char *mnuModify[]={"~Yes, modify to load EGAD.","~No! Do not modify."};
      static char sInstruction[]=
"The INSTALL program can auto-\n\
matically modify your system\n\
files so that EGAD is loaded\n\
each time you turn on or reset\n\
your computer. Or, if you wish\n\
to manually edit your system\n\
files, choose 'No' here.\n";
      static char sInstSorry[]=
"NOTE: INSTALL cannot modify the system files for\n\
this version of DOS.\n";
      static char sInstAlready[]=
"INSTALL has found an existing EGAD load command\n\
on your system. Press <Esc> to exit this program,\n\
and check file: ";
      static char sInstCheck[]=
"NOTE: Check your AUTOEXEC.BAT\n\
and move EGAD above any menuing\n\
systems (WIN, DOSSHELL...)\n";
      static char sInstComplete[]=
"INSTALL operation complete.\n";

      int choice;
      int iSystemDetected;
      int nFileModify;
      char *sBootDisk;

      if ((nFileModify = nFindBestSysMod()) < 0) {
#if 1
	 show_error(sInstSorry,10,15,1);
#else
	 instruction(sInstSorry,10,15);
	 while (gtkey() != 27) {}
	 erase_instruction();
#endif
	 return;
      }
      instruction(sInstruction,10,5);
      choice = menu("Automatically modify your system?",mnuModify,2,10,40);
      erase_instruction();
      if (choice == 0) {	/* Yes... */
	 /* Ask whether boots from A: or C: */
	 if ((sBootDisk = sAskWhichBoots()) != NULL) {
	    /* See if it's already there. */
	    iSystemDetected = iDetectSystem(sBootDisk);
	    if (iSystemDetected & DETECT_EGAD) {
	       strcpy(msTemp,sInstAlready);
	       strcat(msTemp,sSystemFile[(iSystemDetected & DETECT_EGAD_CONFIG)?1:0]);
	       strcat(msTemp,"\n");
	       instruction(msTemp,10,15);
	       while (gtkey() != 27) {}
	       erase_instruction();
	       return;
	    } else {
	       if (bAskModifySpecific(sBootDisk,nFileModify)) {
		  if (iModifySystem(sEgadPath,sBootDisk,nFileModify)) {
		     /* If a menu system was detected, and we modified */
		     /* AUTOEXEC.BAT, notify the user to check to ensure */
		     /* EGAD is loaded before menu system. */
		     if (iSystemDetected & (DETECT_MENU_SYSTEM + DETECT_EGAD_CONFIG) ==
			(DETECT_MENU_SYSTEM)) {
			show_error(sInstCheck,10,15,0);
		     }
		     show_error(sInstComplete,10,-1,0);
		  } else {
		     show_error("INSTALL could not add EGAD load command.",10,15,1);
		  }
	       } else{
		  show_error(sInstComplete,10,-1,0);
	       }
	    }
	 }
      } else {
	 show_error(sInstComplete,10,-1,0);
      }
   }

int check_dup(char *sDirName, char *sEgadFile, int bAllowAdditional)
   {
      FILE *pfTemp;
      static char sInstExists0[]= "A copy of ";
      static char sInstExists1[]= 
" already\n\
exists in this directory.\n\
 If you are installing an\n\
update, choose 'Overwrite.'\n\
 To support a second\n\
printer, use 'Additional.'\n";

      static char *mnuReplace[]={
	 "~NO: Exit (do not modify existing copy)",
	 "~YES: Overwrite existing copy (Update)",
	 "YES: Install an ADDITIONAL copy of EGAD"};
      int retval=1;		/* default to overwriting */
      char sEgadPath[80];

      /* Check to see if it already exists... */
      strcpy(sEgadPath,sDirName);
      strcat(sEgadPath,sEgadFile);
      strcat(sEgadPath,sExeExtension);
      pfTemp = fopen(sEgadPath,"r");
      if (pfTemp != NULL) {
	 fclose(pfTemp);
	 strcpy(msTemp,sInstExists0);
	 strcat(msTemp,sEgadFile);
	 strcat(msTemp,sInstExists1);
	 instruction(msTemp,12,3);
	 retval = menu("Continue installation?",mnuReplace,bAllowAdditional?3:2,12,37);
	 if (retval < 0) retval = 0;
	 erase_instruction();
      }
      return(retval);
   }

/* Returns 0 (success) or an error code. */
/* If an error code, msTemp[] contains the error message. */

int do_copy(char *instfrom,char *sEgadPath,int nPrinterType)
   {
      static char *saExtractMsg[]={
	 "File not found in Archive",
	 "File I/O Error",
	 "File not in Archive format"};
      FILE *infile, *outfile;
      char sArchiveName[64];
      char sDriver[20];
      int iError=0;
      static char sInstruction[]="Please wait for file to be extracted:\n\n";

      strcpy(sArchiveName,instfrom);
      strcat(sArchiveName,"__EGAD__.EX_");
      strcpy(sDriver,driver_name[nPrinterType]);
      strcat(sDriver,sExeExtension);
      strlwr(sDriver);
      if ((infile = fopen(sArchiveName,"rb")) == NULL) {
	 strcpy(msTemp,"Error opening archive ");
	 strcat(msTemp,sArchiveName);
	 iError = -1;
      } else {
	 if ((outfile = fopen(sEgadPath,"wb")) == NULL) {
	    strcpy(msTemp,"Error opening output ");
	    strcat(msTemp,sEgadPath);
	    iError = -1;
	 } else {
	    instruction(sInstruction,10,20);
	    iError = extract(infile,outfile,sDriver);
	    erase_instruction();
	    if (iError) {
	       strcpy(msTemp,"Error installing ");
	       strcat(msTemp,sDriver);
	       strcat(msTemp,"\n  \001");
	       strcat(msTemp,saExtractMsg[iError-1]);
	    }
	 }
      }
      if (iError < 0) {
	 strcat(msTemp,":\n   ");
	 strcat(msTemp,_strerror(NULL));
      }
      if (iError) {
#if 0
	 strcat(msTemp,"\n");
#endif
      } else {
	 AskIfModify(sEgadPath);
	 msTemp[0]='\0';
      }
      return(iError);
   }

int copy_to(char *instfrom, int nPrinterType, char *sDirName, char *sEgadFile)
   {
      int bInstall;
      static char sInst0[] = "An additional copy of EGAD, support for:\n  ";
      static char sInst1[] = "\nwill be installed into directory ";
      static char sInst2[] = "\nThe additional file will be ";
      static char sInst3[] = ".EXE\nTo load, use the command \002 ";
      static char sInst4[] = 
" /T \001\n\nPress <ENTER> to proceed with installation,\n\
or <Esc> to exit.\n";
      char sDirFull[80];
      char sEgadPath[80];
      int akey;

      strcpy(sDirFull,sDirName);
      if (sDirFull[strlen(sDirFull)-1] != '\\') {
	 strcat(sDirFull,"\\");
      }
      bInstall = check_dup(sDirFull,sEgadFile,1);
      if (bInstall == 2) {	/* Install additional... */
	 sEgadFile = driver_name[nPrinterType];
	 /* This time force just Y/N answer */
	 strcpy(msTemp,sInst0);
	 strcat(msTemp,driver_key[nPrinterType]);
	 strcat(msTemp,sInst1);
	 strcat(msTemp,sDirFull);
	 strcat(msTemp,sInst2);
	 strcat(msTemp,sEgadFile);
	 strcat(msTemp,sInst3);
	 strcat(msTemp,sEgadFile);
	 strcat(msTemp,sInst4);

	 instruction(msTemp,8,15);
	 bInstall=0;
	 do {
	    akey = gtkey();
	 } while (akey != 10 && akey != 27);
	 erase_instruction();
	 if (akey == 10) {
	    bInstall = check_dup(sDirFull,sEgadFile,0);
	 }
      }
      if (bInstall) {
	 strcpy(sEgadPath,sDirFull);
	 strcat(sEgadPath,sEgadFile);
	 strcat(sEgadPath,sExeExtension);
	 akey = do_copy(instfrom,sEgadPath,nPrinterType);
	 if (akey) {
	    show_error(msTemp,10,15,1);
	    return(0);
	 }
      }
      return(1);
   }

int make_path_list(char *saPath[])
   {
      char *sPath, *sSearch;
      static char sPathStr[256];
      int iPaths=0;

      if ((sPath = getenv("PATH")) != NULL) {
	 strcpy(sPathStr,sPath);
	 sPath = sPathStr;
	 do {
	    if ((sSearch = strchr(sPath,';')) != NULL) {
	       /* Found ';' -- save one path string */
	       *sSearch = '\0';
	       /* Avoid blank directory names -- (e.g., from "C:\;;C:\SYS") */
	       if (strlen(sPath)) {
		  saPath[iPaths++] = sPath;
	       }
	       sPath = sSearch+1;	/* skip over NUL (previously ';') */
	    }
	 } while (sSearch != NULL);
	 saPath[iPaths++]=sPath;
      }
      return(iPaths);
   }

int install_egad(char *instfrom, int nPrinterType)
   {
      char *saPath[40];
      char sCurDir[65];
      static char sRisaDir[]="C:\\RISA\\NUL";
      int iPaths=0;
      FILE *pfTemp;
      static char sInstBegin[]=
"Select the directory where\n\
you would like to install\n\
EGAD Screen Print.\n\
 To install to a directory\n\
not listed here, press <Esc>\n\
to exit; use the \002 CD \001 command\n\
to go to that directory; then\n\
re-run INSTALL.\n";

      strcpy(msTemp,sInstBegin);
      pfTemp = fopen(sRisaDir,"r");
      if (pfTemp != NULL) {
	 fclose(pfTemp);
	 sRisaDir[7] = '\0';	/* remove '\NUL' */
	 saPath[iPaths++]=sRisaDir;
	 strcat(msTemp," Recommended directory:\n   ");
	 strcat(msTemp,sRisaDir);
	 strcat(msTemp,"\n");
      }
      if (getcwd(sCurDir,127) != NULL) {
	 saPath[iPaths++]=sCurDir;
      }
      iPaths += make_path_list(&saPath[iPaths]);
      instruction(msTemp,12,5);
      iPaths = menu("Select Directory",saPath,iPaths,10,40);
      erase_instruction();
      if (iPaths >= 0) {
	 /* Install EGAD... */
	 copy_to(instfrom,nPrinterType,saPath[iPaths],sEgad);
	 return(1);
      } else {
	 return(0);
      }
   }

char *mnuInstallFrom[] = {"A:","B:"};
char *mnuInstallVerify[]={"Install EGAD","Exit without installing"};
char *mnuInstallTo[20];

main (int argc,char *argv[])
   {
      char instfrom[65];
      int choice;
      int nPrinterType;
      char *p;
      static char basic_inst[] =
"Use arrow keys to move\n\
highlight bar, then\n\
press ENTER to select.\n";

      init_keys();
      set_rows(25);		/* Force alphanumeric mode */
      cursize(1);			/* Set underline cursor */
      cls();
      box(0,10,5,70,2);
      strcpy(instfrom,egad_name);
      strcat(instfrom," Installation");
      center_text(instfrom,1);
      center_text(VERSION,2);
      center_text("LS Software,  8139 E. Mawson Rd,  Mesa, AZ 85207",3);
      center_text("Telephone 602/380-9175  8am-5pm (MST)",4);

      if ((argv[0] != NULL) && (argv[0][0] != '\0') &&
	  (argv[0][1] == ':') && (argv[0][2] == '\\')) {
	 strcpy(instfrom,argv[0]);
	 p = strrchr(instfrom,'\\');
	 if (p != NULL) {
	    *(++p) = '\0';	/* Leave only the path */
	 }
      } else {
	 instruction(basic_inst,12,5);
	 choice = menu("Install EGAD from:",mnuInstallFrom,2,10,40);
	 erase_instruction();
	 switch (choice) {
	    case 0:
	       strcpy(instfrom,"A:\\");
	       break;
	    case 1:
	       strcpy(instfrom,"B:\\");
	       break;
	    default:
	       cls();
	       exit_keys();
	       exit(1);
	 }
      }
      instruction("Please wait...\n",10,33);
      if (read_compat(instfrom) == 0) {
	 show_error("Cannot read Printer Compatibility File.",10,15,1);
	 exit(1);
      }
      erase_instruction();
      instruction(basic_inst,12,5);
      choice = menu("Do you wish to...",mnuInstallVerify,2,10,40);
      erase_instruction();
      switch (choice) {
	 case 0:
	    while (choice == 0) {
	       instruction("Select your printer\n(or a compatible model)\nand press <Enter>.\nOr press <Esc> to exit\nwithout installing.\n",13,5);
	       choice = menu("Printer Menu",cpPrinterName,nprinters,8,35);
	       erase_instruction();
	       if (choice >= 0) {
		  nPrinterType=((struct printer *)cpPrinterName[choice])->driver_type;
		  box(10,15,17,65,2);
		  wstr("You have selected the printer:");
		  setcur(12,20);
		  wstr(cpPrinterName[choice]);
		  setcur(13,17);
		  wstr("This printer will use the EGAD setup for:\001");
		  setcur(14,20);
		  wstr(driver_key[((struct printer *)cpPrinterName[choice])->driver_type]);
		  setcur(15,17);
		  wstr("\002Press <ENTER> to proceed with installation,");
		  setcur(16,20);
		  wstr("or <Esc> to go back to Printer menu: ");
		  do {
		     choice = gtkey();
		     switch (choice) {
			case 27:
			   choice = 0;
			   break;
			case 13:
			case 10:
			   choice = -1;
			   break;
		     }
		  } while (choice > 0);
		  erase(10,15,17,65);
		  if (choice == -1) {
		     choice = install_egad(instfrom,nPrinterType);
		  }
	       }
	    }
	    break;
	 case 1:
	    break;
      }

      exit_keys();
      exit(0);
   }
