/*

	INTFACE.C -- Standard 'C' Screen Interface

	Copyright (c) 1989-1993, LS Software

	8139 E. Mawson
	Mesa, AZ 85207
	(602) 380-9175

	All Rights Reserved. No part of this program may be reproduced
	in whole or in part without express permission of LS Software.

*/

#include <dos.h>
#include <conio.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <fcntl.h> 
#include <io.h> 
#include <memory.h> 
#include <string.h> 
#include <ctype.h>
#include "keys.h"
#include "screen.h"
#include "setegad.h"

extern int system_type();		/* 0 if PC, 1 if Z-100 */

static char *Press_Esc="Press ESC to continue";
char *Press_Ret="Press ENTER to continue";

void center_text(char *text, int line)
{ int left,len;
   len = strlen(text);
   left = 40-(len/2);
   setcur(line,left);
   wstr0(text);
}

void inverse_box(int y1, int x1, int y2, int x2)
{
	wchar(1);
	erase(y1,x1,y2,x2);
	box(y1,x1,y2,x2,2);
}

static int ib_y=-1, ib_x, ib_y2, ib_x2;

void instruction(char *inst_text, int y1, int x1)
   {
      unsigned char text[80], *p, *q;
      int lines=0, width=0, x=0;

      p = inst_text;
      for (;;) {
	 if (*p == '\n') {
	    lines++;
	    if (x>width) {	/* Find maximum width */
	       width=x;
	    }
	    x=0;
	 }
	 if (*p == 0) {		/* end of string */
	    if (x>0) lines++;
	    if (x>width) {	/* Find maximum width */
	       width=x;
	    }
	    break;
	 } else {
	    if (*p >= 32) {
	       x++;		/* Add width of printing chars only */
	    }
	 }
	 p++;
      }
      /* Count last line, too, if not counted above: */
      if (inst_text[strlen(inst_text)-1] != '\n') {
	 lines++;
      }
      if (y1 < 0) {
	 lines++;	/* Add additional line for extra message */
	 y1 = -y1;
      }
      ib_y = y1-1;
      ib_y2= y1+lines;
      if (x1 < 0) {
	 ib_x = 40 - width/2 - 1;
	 ib_x2 = 80 - ib_x + 2;
      } else {
	 ib_x = x1;
	 ib_x2= x1+width+2;
      }
      if (ib_x2>79) ib_x2=79;
      inverse_box(ib_y,ib_x,ib_y2,ib_x2);
      p=text;
      q=inst_text;
      while (*q) {
	 if (*q == '\n') {
	    setcur(y1++,ib_x+2); /* Move cursor */
	    *p = '\0';	/* Null-terminate this line */
	    wstr(text);	/* Print it out */
	    p=text;		/* Start next line */
	    q++;
	 } else {
	    *p++=*q++;
	 }
      }
      setcur(y1,ib_x+2); /* Move cursor */
      wchar(2);		/* normal mode */
   }

void erase_instruction()
{
	if (ib_y >= 0) {
	   erase(ib_y,ib_x,ib_y2,ib_x2);
	   ib_y = -1;
	}
}

/* If baderr is true, make user press ESC, else ENTER will do. */
void show_error(char *the_error, int line, int col, int bad_err)
{ int left,len;
  char *message;
  int wait_key;

#if 1
   char sTemp[255];

	strcpy(sTemp,the_error);
	if (sTemp[strlen(sTemp)-1] != '\n') {
	   strcat(sTemp,"\n");
	}
	instruction(sTemp,-line,col);
	if (bad_err) {
	   message = Press_Esc;
	   wait_key = ESC;
	} else {
	   message = Press_Ret;
	   wait_key = NL;
	}
	wchar(1);	/* Also inverse video */
	wstr(message);
	while (gtkey() != wait_key) {}
	wchar(2);	/* Normal video */
	erase_instruction();
#else
	len = strlen(the_error);
	if (len < 20) len=20;
	if (col < 0)
	  left = 40-(len/2);
	else
	  left = col;
	inverse_box(line,left-1,line+3,left+len+2);
	wstr(the_error);
	if (col>=0) setcur(line+2,left+1);	/* Position to show message */
	if (bad_err) {
	   message = Press_Esc;
	   wait_key = ESC;
	} else {
	   message = Press_Ret;
	   wait_key = NL;
	}

	if (col<0) {
	   center_text(message,line+2);
	} else {
	   wstr(message);
	}
	while (gtkey() != wait_key) ;

	wchar(2);	/* Normal video */
	erase(line,left-2,line+3,left+len+2);
#endif
}

void highlight(char *text, int row, int col, int mode)
   {
      char *found, before[40], hilight, after[40];


      strcpy(before, text);
      found = strchr(before, '~');
      if (found == NULL) {
	 after[0] = hilight = '\0';
      } else {
	 hilight = *(found + 1);	/* char. after '~' is hilighted */
	 strcpy(after, found+2);	/* "after" the hilight */
	 *found = '\0';			/* end "before" string */
      }

      if (row > 0) {
	 setcur(row,col);
      }
      switch (mode) {
	 case 0:		/* Normal */
	    wchar(2);		/* normal */
	    wstr(before);
	    if (hilight) {
	       wchar(4);		/* normal hilight */
	       wchar(hilight);
	       wchar(2);		/* back to normal */
	       wstr(after);
	    }
	    break;
	 case 1:		/* Highlighted */
	    wchar(1);		/* reverse */
	    wstr ( before );
	    wchar(3);		/* reverse hilight */
	    wchar(hilight);
	    wchar(1);		/* reverse */
	    wstr(after);
	    wchar(2);		/* restore normal */
	    break;
	 default:		/* Nothing highlighted */
	    wchar(2);
	    wstr(before);
	    if (hilight) {
	       wchar(hilight);
	       wstr(after);
	    }
	    break;
      }
   }

#define MAXSHOWN 10

int menu(char *sTitle,char *saMenuItems[],int iChoices,
	 int iRowTop, int iColLeft)
   {  int iWidth=0;
      int i,l;
      int c;
      int nChoice=0;
      int nChoicePrev=-1;
      int nChoiceTop=0;
      int iChoicesVisible;
      int bRedraw=1;
      int bDone;	/* 'Done' flag (also 'Scrollable' at first) */

      char *cpHotKey;

      iWidth=strlen(sTitle);
      iChoicesVisible=iChoices;
      bDone = iChoicesVisible > MAXSHOWN;
      if (bDone) {
	 iChoicesVisible = MAXSHOWN;
      }
      /* Future code allows default setting */
      if (iChoices > 128) {
	 nChoice = iChoices >> 8;
	 iChoices &= 128;
      }
      for (i=0; i < iChoices; i++) {
	 l = strlen(saMenuItems[i]);
	 if (strchr(saMenuItems[i],'~') != NULL) {
	    l--;
	 }
	 if ((l) > iWidth) {
	    iWidth = l;
	 }
      }
      box(iRowTop,iColLeft,iRowTop+iChoicesVisible+3,iColLeft+iWidth+2,1);
      setcur(iRowTop+1,iColLeft+1);
      wstr(sTitle);
      setcur(iRowTop+2,iColLeft);
      wchar(0xC6);
      for (i=0; i <= iWidth; i++) {
	 wchar(0xCD);
      }
      wchar(0xB5);
      if (bDone) {
	 setcur(iRowTop+iChoicesVisible+3,iColLeft+iWidth-5);
	 wstr("\x12 more");
      }
      bDone=0;
      while (!bDone) {
	 if (bRedraw) {
	    erase(iRowTop+3,iColLeft+1,iRowTop+iChoicesVisible+2,iColLeft+iWidth);
	    nChoicePrev = -1;		/* No 'previous' choice shown */
	 }
	 for (i=nChoiceTop; i < nChoiceTop + iChoicesVisible; i++) {
	    if ((i == nChoicePrev) || (i == nChoice) || (bRedraw)) {
	       highlight(saMenuItems[i],
		  i-nChoiceTop+iRowTop+3,iColLeft+1,
		  (i == nChoice)?1:0);
	    }
	 }
	 bRedraw=0;
	 setcur(nChoice-nChoiceTop+iRowTop+3,iColLeft+1+strlen(saMenuItems[nChoice]));
	 nChoicePrev=nChoice;
	 c = gtkey();
	 switch (c = toupper(c)) {
	    case ' ':
	    case DOWN:
	    case TAB:
	       nChoice++;
	       break;
	    case PGDN:
	       nChoice += MAXSHOWN;
	       break;
	    case HOME:
	       nChoice=0;
	       break;
	    case BS:
	    case UP:
	    case SH_TAB:
	       nChoice--;
	       break;
	    case PGUP:
	       nChoice -= MAXSHOWN;
	       break;
	    case END:
	       nChoice=iChoices-1;
	       break;
	    case NL:
	       bDone++;
	       break;
	    case ESC:
	       bDone--;
	 }
	 if (isalnum(c)) {
	    for (i=0; i<iChoices; i++) {
	       cpHotKey = strchr(saMenuItems[i],'~');
	       if (cpHotKey != NULL) {
		  if (toupper(*(cpHotKey+1)) == c) {
		     nChoice = i;
		     bDone++;
		     break;
		  }
	       }
	    }
	 }
	 if (nChoice < 0) nChoice=0;
	 if (nChoice >= iChoices) nChoice = iChoices-1;
	 if (nChoice < nChoiceTop) {
	    bRedraw=1;
	    nChoiceTop = nChoice;
	 }
	 if (nChoice > nChoiceTop+iChoicesVisible-1) {
	    bRedraw=1;
	    nChoiceTop = nChoice-iChoicesVisible+1;
	 }
      }
      /* Remove the menu: */
      wchar(2);			/* Normal video */
      erase(iRowTop,iColLeft,iRowTop+iChoices+3,iColLeft+iWidth+2);
      return(bDone>0?nChoice:-1);	/* return -1 if ESC pressed */
   }

int get_int_key;			/* Key pressed to get out of GET_INT */

int get_int(int init_val, int exit_on_arrows)
{ int temp=0;
  int ch=0;
  char buf[10];

  wchar(1);
  wstr0("\040\040\040");		  /* Highlight field */
  while ((ch=gtkey()) != ESC && ch != NL) /* Stop if Enter or ESC pressed */
    {	if (isdigit(ch) && temp <= 99)
	  { temp *= 10; temp += ch - '0'; }
	if (ch == BS)
	  temp /= 10;
	sprintf(buf,"%3d",temp);
	wstr0(buf);
	if (exit_on_arrows &&
	  (ch==UP||ch==DOWN||ch==TAB||ch==SH_TAB||ch==RIGHT||ch==LEFT))
		break;
    }
  get_int_key = ch;			/* This key got us out */
  if (ch == ESC)
    temp=init_val;
  wchar(2);
  sprintf(buf,"%3d",temp);
  wstr0(buf);		/* Un-highlight */
  return(temp);
}

