/* 
	KEYS.H

	Keyboard & function codes

	Copyright (c) 1990 Lindley Systems
*/

/* ASCII codes */
#define CTL_C		0x03
#define BEL		0x07
#define BS		0x08
#define TAB		0x09
#define NL		10
#define CR		13
#define CAN		127
#define CTL_Z		26

/* Keyboard codes */

#define ESC		0x1B	/* Escape key */
#define DELETE		0x7F	/* Delete on Z-100, Ctrl-Backspace on PC */

/* Function Keys */

#define F0		0x96	/* (n/a on PC) */
#define F1		0x97
#define F2		0x98
#define F3		0x99
#define F4		0x9A
#define F5		0x9B
#define F6		0x9C
#define F7		0x9D
#define F8		0x9E
#define F9		0x9F
#define F10		0xA0
#define F11		0xA1
#define F12		0xA2

/* Shifted function keys */

#define SF0		0xD6	/* (n/a on PC) */
#define SF1		0xD7
#define SF2		0xD8
#define SF3		0xD9
#define SF4		0xDA
#define SF5		0xDB
#define SF6		0xDC
#define SF7		0xDD
#define SF8		0xDE
#define SF9		0xDF
#define SF10		0xE0
#define SF11		0xE1
#define SF12		0xE2	/* n/a on Z-100; reserved by DOS for PrtScrn */

/* Control function keys (n/a on Z-100) */

#define CF1		0x80
#define CF2		0x81
#define CF3		0x82
#define CF4		0x83
#define CF5		0x84
#define CF6		0x85
#define CF7		0x86
#define CF8		0x87
#define CF9		0x88
#define CF10		0x89
#define CF11		0x8A
#define CF12		0x8B

/* Alt- function keys (n/a on Z-100) */

#define AF1		0xC0
#define AF2		0xC1
#define AF3		0xC2
#define AF4		0xC3
#define AF5		0xC4
#define AF6		0xC5
#define AF7		0xC6
#define AF8		0xC7
#define AF9		0xC8
#define AF10		0xC9
#define AF11		0xCA
#define AF12		0xCB

/* Editing keys */

#define INS		0xA3	/* I Chr on Z-100, Ins on PC */
#define INS_LINE	0xA4	/* InsLine on Z-100, n/a on PC */
#define UP		0xA5	/* Arrow keys */
#define DOWN		0xA6
#define RIGHT		0xA7
#define LEFT		0xA8
#define HOME		0xA9	/* Home on Z-100 and PC */
#define DEL		0xE3	/* D Chr on Z-100, Del on PC */
#define DEL_LINE	0xE4	/* DelLine on Z-100, n/a on PC */
#define SH_UP		0xE5	/* Shift-Arrows on Z-100, n/a on PC */
#define SH_DOWN		0xE6
#define SH_RIGHT	0xE7
#define SH_LEFT		0xE8
#define END		0xE9	/* Shift-Home on Z-100, End on PC */

/* Aliases for clarity on PC */

#define PGUP		SH_UP
#define PGDN		SH_DOWN
#define CTL_LEFT	SH_LEFT
#define CTL_RIGHT	SH_RIGHT

/* Z-100 Keypad, not available on PC */

#define K0		0xB0
#define K1		0xB1
#define K2		0xB2
#define K3		0xB3
#define K4		0xB4
#define K5		0xB5
#define K6		0xB6
#define K7		0xB7
#define K8		0xB8
#define K9		0xB9

/* Shifted Z-100 Keypad, and Ctrl-Num pad on PC */

#define SK0		0xF0
#define SK1		0xF1	/* Ctrl-End on PC */
#define SK2		0xF2
#define SK3		0xF3	/* Ctrl-PgDn on PC */
#define SK4		0xF4
#define SK5		0xF5
#define SK6		0xF6
#define SK7		0xF7	/* Ctrl-Home on PC */
#define SK8		0xF8
#define SK9		0xF9	/* Ctrl-PgUp on PC */
#define SKDASH		0xED	/* Shift-Kpd Dash on Z-100, Alt-Hyphen on PC */
#define SKDOT		0xEE	/* Shift-Kpd Dot on Z-100, Alt-= on PC */

#define CTL_END		0xF1	/* Ctrl-End on PC */
#define CTL_PGDN	0xF3	/* Ctrl-PgDn on PC */
#define CTL_HOME	0xF7	/* Ctrl-Home on PC */
#define CTL_PGUP	0xF9	/* Ctrl-PgUp on PC */

#define SH_TAB		0xFA	/* Shift-Tab on PC, n/a on Z-100 */

/* Notes:

	HELP key on Z-100 is mapped to F1 key by logic in ZCON.C.

	The following keys are not available on the PC version:

		"Alt" combinations including Alt-A through Alt-Z,
			Alt-0 through Alt-9, Alt-Hyphen, Alt-=
*/
