#include <stdio.h>
#ifndef __TURBOC__
#include <graph.h>
#endif
#include <process.h>
#include <malloc.h>
#include "setegad.h"

extern char prog_name[];
extern int which_printer;	/* 0..NPRINTERS-1 */
extern PRINTER_DEF *characteristics[];
extern char egad_name[];
extern PAR_T *working_parms;

extern int setting_disk;
extern int in_mem;		/* -1 if no driver in mem; else which_printer */

struct box {
	int x1, y1, x2, y2;
};

static int scrnprint(struct box *boundary, int *colortable)
{ char cmd_string[132];
  char anum[10];
  int i;
  int err;

    sprintf(cmd_string,"/P/B0/R=%d,%d,%d,%d/A3/O1/E1",
      boundary->x1, boundary->y1, boundary->x2, boundary->y2);
    if (colortable != NULL) {
       strcat(cmd_string,"/F0/C=");
       for (i=0; i<=15; i++) {
	  sprintf(anum,"%d ",colortable[i]);
	  strcat(cmd_string,anum);
       }
    } else {
       strcat(cmd_string,"/F1");
    }
    if ((working_parms->unit) > 0) {
       sprintf(anum,"/D=LPT%1d",working_parms->unit);
    } else {
       sprintf(anum,"/D=COM%1d",-working_parms->unit);
    }
    strcat(cmd_string,anum);
#if 1
    err=spawnlp(P_WAIT, prog_name, prog_name, cmd_string, NULL);
#else
    _settextposition(15,0);
    printf(cmd_string);
    getch();
    err=0;
#endif
    return(err);
}

static int formfeed()
{
    int err;
    char cmd_string[80];
    char anum[10];

    strcpy(cmd_string,"/P/F=2/B=0");
    if (working_parms->unit < 0) {
       sprintf(anum,"/D=COM%1d",-working_parms->unit);
    } else {
       sprintf(anum,"/D=LPT%1d",working_parms->unit);
    }
    strcat(cmd_string,anum);
#if 1
    err=spawnlp(P_WAIT, prog_name, prog_name, cmd_string, NULL);
    return(err);
#endif
}

int sample(int prn_id)
{
#ifndef __TURBOC__
	int colors[16];
	struct box rbox;
	int colorloop;
	int acolor;
	int black, white;
	int maxcolor;
	char title[80];

	white = characteristics[which_printer]->printer_palette[1][0];
	black = characteristics[which_printer]->printer_palette[1][9];
	/* add 7 to round up when dividing by 8 */
	maxcolor = valid_color(prn_id,1,-99);	/* Find max color # */
	maxcolor += 7;				/* So we always print it */

	if (_setvideomode(_ERESCOLOR) ==0) {
	   return(-1);
	}
	_settextcolor(7);
	/* Actually since we don't use _outtext, all text comes out in */
	/* color 7 anyway! */
	_settextposition(1,35);
	printf("Color Chart");
	_settextposition(2,25);
	printf("For use with %s",egad_name);
	strcpy(title,"With ");
	strcat(title,characteristics[which_printer]->Printer_Name);
	_settextposition(3,40-strlen(title)/2);
	printf(title);
	memset(colors,white,16 * sizeof(int));	/* set all to white */
	colors[7]=black;			/* black */
	rbox.x1 = 0;
	rbox.y1 = 0;
	rbox.x2 = 640;
	rbox.y2 = 64;
	scrnprint(&rbox,&colors[0]);
	rbox.x2 = 256+32;
	rbox.y2 = 38;
	_clearscreen(_GCLEARSCREEN);
	_settextposition(12,0);
	printf("You may hear several beeps as printing progresses;\n");
	printf("this is normal.");
	for (colorloop=0; colorloop < maxcolor / 8; colorloop++) {
	   for (acolor = 0; acolor < 8; acolor++) {
	      _settextposition(0,acolor*4+3);
	      printf("%2d",colorloop*8+acolor);
	      _setcolor(acolor+8);
	      _rectangle(_GFILLINTERIOR,acolor*32+10,14,acolor*32+36,28);
	      colors[acolor+8]=colorloop*8+acolor;
	   }
	   _settextposition(12,0);
	   scrnprint(&rbox,&colors[0]);
	}
	formfeed();
	_setvideomode(_DEFAULTMODE);
	return(0);
#endif
}

int iDrawTestPattern(int vmode)
   {
#ifndef __TURBOC__
      struct videoconfig gconfig;	/* Current graphics configuration */
      int xmax, ymax;
      int maxcolor;
      int textheight;
      int i;
      int x0, y0;
      int xsize,ysize;
      static char egadtext[]=
"Four score and seven years ago our forefathers brought forth upon this continent";
      char temp[10];
      struct box rbox;

      if (_setvideomode(vmode)) {
	 _getvideoconfig(&gconfig);

	 xmax = gconfig.numxpixels;
	 ymax = gconfig.numypixels;
	 maxcolor = gconfig.numcolors;
	 if (maxcolor>23) maxcolor=23;
	 textheight = ymax/(gconfig.numtextrows-1);

	 _settextcolor(7);
	 _settextposition(4,2);
	 printf("%d by %d Graphics\n",xmax--,ymax--);
	 printf(" in %d Colors.\n\n",maxcolor);
	 printf(" EGAD Screen Print\n   Sample");
	 /* Now xmax, ymax are, e.g., 0..639 */

	 _settextposition(2,gconfig.numtextcols/3 + 2);
	 printf("Square");
	 _settextposition(3,gconfig.numtextcols/3 + 4);
	 printf("on");
	 _settextposition(4,gconfig.numtextcols/3 + 2);
	 printf("Screen");

	 rbox.x1 = 0;
	 rbox.y1 = 0;
	 rbox.x2 = xmax;
	 rbox.y2 = ymax;

	 _setcolor(7);
	 _rectangle(_GBORDER,0,0,xmax,ymax);
	 _rectangle(_GBORDER,xmax/3,0,xmax/3+xmax/4,ymax/3);
	 for (i=maxcolor-1;i>=0;i--) {
	    _setcolor(i);
	    xsize = (((xmax/4) * (i*3))/maxcolor)/2;
	    ysize = (((ymax/3) * (i*3))/maxcolor)/2;
	    x0 = (xmax/5) - xsize/4;
	    y0 = (ymax/2) - (ysize/4);
	    _ellipse(_GBORDER,x0,y0,x0+xsize,y0+ysize);
	    x0 += (xmax*4)/10;
	    _ellipse(_GFILLINTERIOR,x0,y0,x0+xsize,y0+ysize);

	    _rectangle(_GFILLINTERIOR,xmax-20,(i+1)*textheight+1,xmax-4,(i+2)*textheight);
	    _settextposition(i+2,xmax/8-4);
	    printf("%2d",i);
	 }
	 _settextposition(gconfig.numtextrows-4,1);
	 for (i=1; i<maxcolor; i++) {
	    _settextcolor(i);
	    temp[1]=0;
	    for (x0=0; x0<8; x0++) {
/*	       temp[0]=egadtext[(i*8+x0)%(strlen(egadtext))]; */
	       temp[0]=egadtext[((i-1)*8+x0)%80];
	       _outtext(temp);
	    }
	 }
	 i = getch();
	 if (setting_disk && ((i == 'p') || (i == 'P'))) {
	   scrnprint(&rbox,NULL);
	 }
	 return(1);
      } else {
	 return(0);
      }
#endif
   }


void iDoTestPatterns()
   {
      static char sInstMain[]=
"If you select to proceed now by pressing \002 ENTER \001, several sample\n\
screens will be displayed in the various resolutions you screen can\n\
display. These will allow you to test EGAD's print capabilities such\n\
as color and aspect ratio matching.\n\n\
Press: \002 ENTER \001 Save the current settings; Proceed to samples.\n\
   Or: \002 Esc \001   Do not save settings; do not display samples.\n";
      static char sInstKey0[]=
"Your Settings have been saved.\n\n\
 To advance to the next screen, press any key.\n";
      static char sInstDisk[]=
" Press the \002 P \001 key while any of the sample screens\n\
  are shown to make a printout using the DISK settings.\n";
      static char sInstMem[]=
" Use the \002 Print Screen \001 key to make a printout\n\
  using the memory settings.\n";
      int i;
      char *sMessage;

      cls();
      instruction(sInstMain,8,5);
      do {
	 switch (i = gtkey()) {
	    case 10:	/* ENTER */
	       SaveSettings();
	       break;
	    case 27:	/* Esc */
	       erase_instruction();
	       return;
	 }
      } while (i != 10);
      erase_instruction();
      if ((sMessage = malloc(1024)) != NULL) {
	 strcpy(sMessage,sInstKey0);
	 if (setting_disk) {
	    strcat(sMessage,sInstDisk);
	 }
	 if (in_mem >= 0) {
	    strcat(sMessage,sInstMem);
	 }
	 instruction(sMessage,8,5);
	 free(sMessage);
      }
      i=gtkey();
      erase_instruction();

#ifndef __TURBOC__
/*      iDrawTestPattern(_VRES256COLOR); */
      iDrawTestPattern(_MRES4COLOR);
      iDrawTestPattern(_MRES16COLOR);
      iDrawTestPattern(_ERESCOLOR);
      iDrawTestPattern(_VRES16COLOR);
      iDrawTestPattern(_MRES256COLOR);
/*      iDrawTestPattern(_HERCMONO); */
      _setvideomode(_DEFAULTMODE);
#endif
   }
