/*

	SETEGAD.C -- Set EGAD Screen Print parameters

	EGAD

	Copyright (c) 1989, 1990, 1991 LS Software

	8139 E. Mawson
	Mesa, AZ 85207
	(602) 380-9175

	All Rights Reserved. No part of this program may be reproduced
	in whole or in part without express permission of LS Software.

*/

#include <dos.h>
#include <conio.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <fcntl.h> 
#include <io.h> 
#include <memory.h> 
#include <string.h> 
#include "keys.h"
#include "screen.h"
#include "setegad.h"
#include "version.h"

extern PRINTER_DEF *characteristics[];

extern int get_int_key;

extern int which_printer;	/* 0..NPRINTERS-1 */
extern int setting_disk;	/* 0 if not */
extern int setting_mem;		/* 0 if not */
extern int mem_version;		/* VER_* */
extern int disk_version;	/* VER_* */

extern char *Color_Name[];
extern char *Text_Name[];

extern PAR_T *working_parms;

void color_num(int number, int display_as, int value, int row, int col)
   {  char buf[8];

      setcur(row,col);
      color(display_as==0?0x70:display_as);
      sprintf(buf,"%2d",number);
      wstr(buf);
      wchar(2);		/* Normal attribute */
      sprintf(buf,":%3d",value);
      wstr(buf);
   }

int cga4[4]={0,3,5,7};
int cga2[2]={0,7};

void show_colors(ptr)
PAR_T *ptr;
   {  int i;

      if (!system_type())		/* on PC, show four palettes: */
	 { setcur(6,1);
	 wstr("\1Colors:   EGA/VGA   CGA      CGA/MGA \2");
	 setcur(7,4);
	 wstr(      "Text   16-Color  4-Color  2-Color");
	 for (i=0; i<16; i++) { 
	    /* If we print text backgrounds, go 0..7, 0..7; else 0..15 foregnds */
	    if (characteristics[which_printer]->printer_text_mode&P_TEXT_BG_COLOR) {
	       setcur(i+8,1);
	       if (i==0) wstr("Fg");
	       if (i==8) wstr("Bg");
	       color_num(i&7,i==0?0x70:(i>7?((i&7)<<4)|0x0f:i),
		  ptr->palette_text[i],i+8,4);
	    } else {
	       color_num(i,i,ptr->palette_text[i],i+8,4);
	    }

	    color_num(i,i,ptr->palette_16[i],i+8,13);
	    if (i<4) color_num(i,cga4[i],ptr->palette_4[i],i+8,22);
	    if (i<2) color_num(i,cga2[i],ptr->palette_2[i],i+8,31);
	 }
	 setcur(20,22);
	 wstr("VGA/256 Divisor:");
	 setcur(21,30);
	 printf("%3d",ptr->vga_divisor);
#if 0
      } else { 	/* Z-100: */
	 setcur(7,13);
	 wstr("\1Colors:\2");
	 for (i=0; i<8; i++) {
	    color_num(i,i,ptr->palette_16[i],i+8,13);
	 }
#endif
      }
   }

void main_message()
   {  static char *msg="Select SCREEN color using the UP/\n\
DOWN arrows, SPACE, or BACKSPACE;\n\
then press ENTER to change that\n\
color.  Press ESC to exit.\n";

      erase_instruction();
      instruction(msg,10,42);
   }

void entry_msg(int which_entry, int cur_val)
   {  static char *msg="Enter the PRINTER color for\n\
SCREEN color %d and press ENTER;\n\
or press ESC to restore value to %d\n";
      char xmsg[132];

      sprintf(xmsg, msg, which_entry, cur_val);
      erase_instruction();
      instruction(xmsg, 10, 42);

   }

int set_some(int prn_id, int grafx, int *array, int array_len, int xpos, int ypos)
   {
      int ch=0, i=0, newmsg=1, temp, retval=0;

      while(ch >= 0) {	
	 if (newmsg) {
	    main_message();
	    newmsg=0;
	 }
	 setcur(ypos+i,xpos+5);
	 wchar(16);
	 setcur(ypos+i,xpos+5);
	 cursize(2);			/* Block cursor */
	 if (ch==0) ch = gtkey();	/* If ch is 0, we get user input */
	 wchar(':');
	 cursize(1);			/* underline cursor */
	 switch(ch) { 
	    case UP:
	    case BS:
	       i--;
	       if (i<0) i = array_len-1;
	       ch = 0;
	       break;
	    case DOWN:
	    case ' ':
	       i++;
	       if (i>=array_len) i=0;
	       ch = 0;
	       break;
	    case LEFT:
	    case SH_TAB:
	       if (!system_type()) { 
		  retval = -1;	/* go LEFT */
		  ch = -1;
	       } else {
		  ch = 0;
	       }
	       break;
	    case RIGHT:
	    case '\t':
	       if (!system_type()) {
		  retval = 1;		/* go RIGHT */
		  ch = -1;
	       } else {
		  ch = 0;
	       }
	       break;
	    case NL:
	       newmsg=1;
	       entry_msg(i,array[i]);
	       setcur(ypos+i,xpos+6);
	       temp=get_int(array[i],1);
	       if (!characteristics[which_printer]->other_colors) {
		  /* Check for valid # */
		  int newtemp=-1, x, *p;
		  p = characteristics[which_printer]->printer_palette[grafx];
		  for (x=0; x<10; x++) {
		     if (temp == *p++) {
			newtemp = temp;
		     }
		  }
		  temp = newtemp;
	       }
	       if (valid_color(prn_id,grafx,temp)) {
		  array[i] = temp;
	       }
	       setcur(ypos+i,xpos+6);
	       printf("%3d",array[i]);
	       ch = get_int_key;
	       if (ch == NL || ch == ESC) {
		  ch=0;			/* If we pressed NL or ESCto get out, */
		  /* get more input right away. Else */
		  /* force the key the user pressed. */
	       }
	       break;
	    case ESC:
	       ch= -1;
	       break;
	    default:
	       ch = 0;			/* Can't handle it, get more input */
	 }
      }
      erase_instruction();
      return(retval);
   }

/* grafx is 0 for Text palette, 1 for Graphics palette */

void show_color_palette(grafx)
   {
      int i,x,y,posn;
      int *p;
      char **attribute;
      char title[32];

      box(16,42,24,78,1);
      setcur(17,45);
      p = characteristics[which_printer]->printer_palette[grafx];
      sprintf(title,"\1Printer %s Color Numbers:\2",grafx?"Graphics":"Text");
      wstr(title);
      if (!grafx && 
	    (characteristics[which_printer]->printer_text_mode&P_TEXT_ATTRS)) {
	 attribute = Text_Name;
      } else {
	 attribute = Color_Name;
      }
      posn=0;
      for (i=0; i<10; i++) {
	 y=(posn%5)+18;
	 x=45+((posn/5)*15);
	 setcur(y,x);
	 if (*p >= 0) {		/* if can print this color */
	    printf("%3d =",*p);
	    setcur(y,x+6);
	    wstr(attribute[i]);
	    posn++;
	 }
	 p++;
      }
      if (grafx && (characteristics[which_printer]->other_colors)) {
	 /* More than just the listed colors for this printer?
	  (Graphics mode only) */
	 setcur(23,47);
	 wstr("(See Color Chart for More)");
      }
   }

int valid_color (int prn_id, int grafx, int pcolor)
   {
      int i;
      int maxcolor;
      int *p;

      if (grafx) {
	 /* Graphics colors */
	 maxcolor = (prn_id & TYP_MATRIX)?7:15;	/* Matrix have 7, others 15 */
	 if (prn_id & TYP_COLOR) {		/* Unless color, when 63. */
	    maxcolor = 63;
	 }
      } else {
	 /* Text colors */
	 maxcolor = 7;
      }
      if (pcolor == -99) return (maxcolor);	/* Request for max color # */
      if (pcolor < 0) return(0);		/* Return illegal: won't chg */
      if (pcolor > maxcolor) return(0);		/* Illegal color */
      if (grafx && (characteristics[which_printer]->other_colors)) {
	 return(1);		/* All colors assumed valid */
      }
      p = characteristics[which_printer]->printer_palette[grafx];
      for (i=0; i<10; i++) {	/* Ensure color is listed on menu */
	 if (*p++ == pcolor) {
	    return(1);
	 }
      }
      return(0);
   }

void reverse_palette(ptr)
PAR_T *ptr;	/* Parameter array */
{ int x, y;
	x=ptr->palette_2[0];
	ptr->palette_2[0]=ptr->palette_2[1];
	ptr->palette_2[1]=x;

	for (y=0; y<2; y++)
	  { x=ptr->palette_4[y];
	    ptr->palette_4[y]=ptr->palette_4[3-y];
	    ptr->palette_4[3-y]=x;
	  }

	for (y=0; y<8; y++)
	  { x=ptr->palette_16[y];
	    ptr->palette_16[y]=ptr->palette_16[15-y];
	    ptr->palette_16[15-y]=x;
	  }
}

void black_and_white_colors(ptr)
PAR_T *ptr;	/* Parameter array */
{ int black, white, i, newcolor;

	white = (characteristics[which_printer])->printer_palette[1][0];
	black = (characteristics[which_printer])->printer_palette[1][9];
	newcolor = white;		/* 0 prints as white */
	for (i=0; i<16; i++)
	  { if (i > 0)
	      newcolor = black;		/* 1..n print as black */
	    ptr -> palette_16[i] = newcolor;
	    if (i < 2)
	      ptr -> palette_2[i] = newcolor;
	    if (i < 4)
	      ptr -> palette_4[i] = newcolor;
	  }
}

void default_colors()
{  int *p, *q;
	
	p = &working_parms->palette_2[0];	/* First integer in color array */
	/* Copy default palette for this printer */
	q = &(characteristics[which_printer]->default_palette)->palette_2[0];
	memcpy(p,q,38*2);		/* Copy 38 integers */
}

char vga_msg[]="\
In VGA and MCGA 256-color modes,\n\
the screen colors will be divided\n\
by this number; the result will\n\
be taken modulo 16 for color index.\n";

char *Color_Menu[]={"Set ~Text Colors","Set ~EGA/VGA Colors",
		    "Set CGA/~4 Colors","Set CGA/MGA/~2 Colors",
		    "~Default Color Table","~Black/White only Table",
		    "~Reverse Color Table",
		    "~VGA/256 Divisor","E~xit to Change Menu"};

char *ZColor_Menu[]={"~Set Color Choices","~Default Color Table",
			"~Black/White only Table","~Reverse Color Table",
			"E~xit to Change Menu"};

int ZColor_Menu_Map[]={1,4,5,6,8};

int sel_colors(ptr)
PAR_T *ptr; /* Parameter array */
   {
      int x,y,next_op=0;

      x = 99;
      while (x >= 0) {
	 if (!next_op) {
	    x = menu("Set Colors Menu",
	       system_type()?ZColor_Menu:Color_Menu,
	       system_type()?5:9,10,42);
	 } else {
	    x += next_op;	/* Buffered user movement command */
	    x=(x<0)?3:(x>3)?0:x;	/* Wrap around */
	 }
	 if (system_type() && x>=0) {
	    x = ZColor_Menu_Map[x];
	 }
	 next_op=0;		/* Assume no buffered operation */
	 switch(x) {
	    case 0:
	       if (characteristics[which_printer]->printer_text_mode == P_TEXT_NONE) {
		  show_error("Not Available on this printer",12,43,0);
	       } else {
		  show_color_palette(0);		/* Show text palette */
		  next_op=set_some(ptr->printer_flag,0,ptr->palette_text,16, 1, 8);
	       }
	       break;
	    case 1:
	       show_color_palette(1);		/* Show graphics palette */
	       next_op=set_some(ptr->printer_flag,1,
		  ptr->palette_16,system_type()?8:16, 10, 8);
	       break;
	    case 2:
	       show_color_palette(1);		/* Show graphics palette */
	       next_op=set_some(ptr->printer_flag,1,
		  ptr->palette_4, 4, 19, 8);
	       break;
	    case 3:
	       show_color_palette(1);		/* Show graphics palette */
	       next_op=set_some(ptr->printer_flag,1,
		  ptr->palette_2, 2, 28, 8);
	       break;
	    case 4:
	       default_colors();
	       break;
	    case 5:
	       black_and_white_colors(ptr);
	       break;
	    case 6:
	       reverse_palette(ptr);
	       x=0;
	       break;
	    case 7:
	       instruction(vga_msg,10,42);
	       setcur(21,30);
	       y = get_int(ptr->vga_divisor,0);
	       if (y>0) {
		  ptr->vga_divisor = y;
	       }
	       erase_instruction();
	       break;
	    case 8:
	       x = -1;
	 }
	 erase(16,42,24,78);
	 show_colors(ptr);
      }

      return(1);
   }

