/*

	SETEGAD.C -- Set EGAD Screen Print parameters

	EGAD

	Copyright (c) 1989, 1990, 1991 LS Software

	8139 E. Mawson
	Mesa, AZ 85207
	(602) 380-9175

	All Rights Reserved. No part of this program may be reproduced
	in whole or in part without express permission of LS Software.

*/

#include <dos.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>
#include <memory.h>
#include <string.h>
/* #include <dir.h> */
#include "keys.h"
#include "screen.h"
#include "setegad.h"
#include "version.h"

extern int system_type();		/* 0 if PC, 1 if Z-100 */
extern PRINTER_DEF *characteristics[];
extern char *Color_Name[];		/* Color names */
extern char *Text_Name[];		/* Text attribute names */
extern int nprinters;

#define VER_INVALID	0	/* Invalid EGAD signature */
#define VER_OLD		1	/* Old-style */
#define VER_NEW		2	/* new style */

#define MAX_X		800
#define MAX_Y		800

static union REGS Regs;

char egad_name[]="EGAD Screen Print";

int which_printer;		/* 0..NPRINTERS-1 */
int in_mem;			/* -1 if no driver in mem; else which_printer */
int setting_disk;		/* 0 if not */
int setting_mem;		/* 0 if not */
int mem_version=0;		/* VER_* */
int disk_version=0;		/* VER_* */
int mem_chg,dsk_chg;		/* TRUE if memory/disk changed */
static char *Mem_or_Disk[]={"Disk","Memory"};

int prog_file;
long par_loc, lcheck;
unsigned int ucheck;
char prog_name[81];
char out_port[6]="LPT1";

char far *prtsc_vec;
union REGS inregs, outregs;
struct SREGS segregs;
int selection,status;

char *basic_inst =
"Use arrow keys to move\n\
highlight bar, then\n\
press ENTER to select.\n";

PAR_T *working_parms;

#define MAXPROGRAMS 32
char *saEgadPrograms[MAXPROGRAMS];
int nEgadPrograms=0;

static char Main_Sel[]  = "Select Operation";
static char *Main_Menu[]={
	"E~xit to DOS",
	"Set Program in ~Disk File",
	"Set Program in ~Memory",
	"Set ~Program in Disk & Memory"};

/*
	GET_SELECTION returns a bit field:

	1	Setting Disk	(0=neither, 3=both)
	2	Setting Memory
*/

int get_selection(int in_mem)
   {
      int t;
      char pname[80];

      strcpy(pname,"EGAD Driver in memory: ");
      strcat(pname,in_mem>=0?characteristics[in_mem]->Printer_Name:"None");
      box(6,10,8,70,1);
      center_text(pname,7);

      instruction(basic_inst,12,45);
      t = menu(Main_Sel,Main_Menu,in_mem<0?2:4,10,10);
      erase_instruction();
      return(t);
   }

/* Given the two Type characters in the driver, return a printer number
   (0 .. NPRINTERS - 1) */

int printer_type (int two_chars)
{ int i;

	for (i=0; i<nprinters; i++)
	  if ((characteristics[i]->signature) == two_chars)
	    return(i);
	return(-1);
}

/*	determines whether the passed structure is a valid EGAD signature
*	returns:
*		VER_INVALID, VER_OLD, or VER_NEW
*	Note! In an effort to stop people from running PKLITE (et al) on
*	EGAD and trying to "set" the *COMPRESSED* versions, this is a little
*	pickier than it used to be.  -- wl 01/20/93
*/
int valid_signature(PAR_T *ptr)
{
   int i;
   if (ptr->identity_flag != EGAD_IDENT)		/* invalid ID */
     return(VER_INVALID);
   /* If number of parameters is > 10, it's a New block.  Accept it. */
   /* Otherwise it's an old one.  We removed the conversion logic, so call it INVALID. */
   if (ptr->signature <= 10) {
      return(VER_INVALID);
   }
   /* Check the number of variables in the header */
   if (ptr->hdr_nvars < 5 || ptr->hdr_nvars > 30) {
      return(VER_INVALID);
   }
   for (i=0; i<15; i++) {
      if (ptr->palette_16[i] > 255 || ptr->palette_text[i] > 255) {
	 return(VER_INVALID);
      }
   }
   return(VER_NEW);
}

char *Error_Msg[]={"open","read","writ","clos"};

static void error_reading_pgm(int error_no)
{  char buf[80];
	sprintf(buf,"Error %sing file %s",Error_Msg[error_no],prog_name);
	show_error(buf,10,-1,1);
}

char Extension[]=".EXE";

int open_pgm() /* Returns file channel #, or -1 if can't find */
{
#ifdef __TURBOC__
  char drive[MAXDRIVE], dir[MAXDIR], fname[MAXFILE], ext[MAXEXT];
  int f;
  char *complete_path;

	strupr(prog_name);
	fnsplit(prog_name,drive,dir,fname,ext);
	fnmerge(prog_name,drive,dir,fname,Extension);

	f = open(prog_name,O_RDWR|O_BINARY);
	if (f >= 0)
	  return(f);	/* Found the program */

	strcpy(prog_name,fname);
	strcat(prog_name,Extension);	/* prog.exe */
	complete_path=searchpath(prog_name);

	if (complete_path != NULL) { /* If file found */
	  strcpy(prog_name,complete_path);
	  return(open(prog_name,O_RDWR|O_BINARY));
	} else {
	  return(-1);
	}
#else
  char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
  int f;
  char complete_path[65];

	strupr(prog_name);
	_splitpath(prog_name,drive,dir,fname,ext);
	_makepath(prog_name,drive,dir,fname,Extension);
	f = open(prog_name,O_RDWR|O_BINARY);
	if (f >= 0)
	  return(f);	/* Found the program */

	strcpy(prog_name,fname);
	strcat(prog_name,Extension);	/* prog.exe */
	_searchenv(prog_name,"PATH",complete_path);

	if (*complete_path) {	/* If file found */
	   strcpy(prog_name,complete_path);
	   return(open(prog_name,O_RDWR|O_BINARY));
	} else {
	   return(-1);
	}
#endif
}

void FreeEgadPgms()
   {
      while (nEgadPrograms--) {
	 if (saEgadPrograms[nEgadPrograms]!=NULL) {
	    free(saEgadPrograms[nEgadPrograms]);
	    saEgadPrograms[nEgadPrograms]=NULL;
	 }
      }
      nEgadPrograms=0;
      saEgadPrograms[0]=NULL;
   }

void AddEgadPgm(char *sPath)
   {
      int nCheck;

      strupr(sPath);
      for (nCheck = 0; nCheck < nEgadPrograms; nCheck++) {
	 if (strcmp(saEgadPrograms[nCheck],sPath)==0) {
	    return;		/* Already exists */
	 }
      }
      if (nEgadPrograms < MAXPROGRAMS) {
	 saEgadPrograms[nEgadPrograms++]=strdup(sPath);
      }
   }

void SearchDirForFile(char *sDir,char *sFile)
   {
      char sFullPath[65];
      struct find_t ftEgadFile;

      strcpy(sFullPath,sDir);		/* Directory */
      /* Be sure path ends with a backslash */
      if (sFullPath[strlen(sFullPath)-1] != '\\') {
	 strcat(sFullPath,"\\");
      }
      strcat(sFullPath,sFile);		/* Plus EGAD or EGAD* */
      strcat(sFullPath,".EXE");		/* Plus .EXE */
      if (_dos_findfirst(sFullPath,_A_NORMAL,&ftEgadFile) == 0) {
	 do {
	    strcpy(sFullPath,sDir);
	    /* Be sure path ends with a backslash */
	    if (sFullPath[strlen(sFullPath)-1] != '\\') {
	       strcat(sFullPath,"\\");
	    }
	    strcat(sFullPath,ftEgadFile.name);
	    AddEgadPgm(sFullPath);
	 } while (_dos_findnext(&ftEgadFile)==0);
      }
   }

int SearchDirForEgad(char *sDir)
   {
      char sEgadDir[65];
      char sEgadName[65];
      char sDefaultName[16];
      char far *mcb_vec;
      int i;

      /* If in memory, First look for the program which the user loaded: */
      if (setting_mem) {
	 FP_SEG(mcb_vec)=working_parms->tsr_psp_seg - 1;
	 FP_OFF(mcb_vec)=8;		/* offset of program name */
	 for (i=0; i<8; i++) {
	    sDefaultName[i] = *mcb_vec++;
	 }
	 sDefaultName[8] = 0;
	 if (strlen(sDefaultName) > 4) {	/* not just EGAD or blank */
	    SearchDirForFile(sDir,sDefaultName);
	 }
      }
      SearchDirForFile(sDir,"EGAD*");
   }

/* Now search: C:\RISA; the current directory; and all the dir's in the PATH */
/* for EGAD*.EXE  and if found call AddEgadPgm(). */
/* Then let the user pick from that list. */

void SearchPathForEgad()
   {
      char *sPath, *sSearch;
      static char sPathStr[256];
      int iPaths=0;

      FreeEgadPgms();
      if (getcwd(sPathStr,80) == NULL) {
	 return;
      }
      SearchDirForEgad(sPathStr);
      if ((sPath = getenv("PATH")) != NULL) {
	 strcpy(sPathStr,sPath);
	 sPath = sPathStr;
	 do {
	    if ((sSearch = strchr(sPath,';')) != NULL) {
	       /* Found ';' -- save one path string */
	       *sSearch = '\0';
	       /* Avoid blank directory names -- (e.g., from "C:\;;C:\SYS") */
	       if (strlen(sPath)) {
		  SearchDirForEgad(sPath);
	       }
	       sPath = sSearch+1;	/* skip over NUL (previously ';') */
	    }
	 } while (sSearch != NULL);
	 SearchDirForEgad(sPath);
      }
   }

int SelectEgadProgram()
   {
      static char sInstNotFound[]=
"Cannot locate EGAD.EXE or any EGAD*.EXE\n\
programs in the current directory, or in\n\
any directory in the \002 PATH \001.\n\
 Use the CD command to change to the\n\
  directory into which you installed EGAD.\n\
 Or run the INSTALL program from the\n\
  EGAD diskette if you have not yet\n\
  installed EGAD.\n\
Press <Esc> to continue.\n";

      SearchPathForEgad();
      if (nEgadPrograms) {
	 return(menu("Select EGAD program file:",saEgadPrograms,nEgadPrograms,10,10));
      } else {
	 instruction(sInstNotFound,10,15);
	 while (gtkey() != 27) {
	 }
	 erase_instruction();
	 return(-1);
      }
   }

#define SEARCH_LEN 512

int get_program(PAR_T *ptr, char *program)
   { 
      int opt, pgm_printer;

      int diff_printer=0;
      static char *diff_msg[]={"Memory","Selected"};

      char buf[80];
      char params[SEARCH_LEN];	/* parameters from disk */
      int  exe_hdr_len;
      char *param_scan;

      /* Determine if program specified on command line.  If not, select one. */
      if (program != NULL) {
	 strcpy(prog_name,program);
      } else {
	 opt = SelectEgadProgram();
	 if (opt < 0) {
	    return(0);
	 } else {
	    strcpy(prog_name,saEgadPrograms[opt]);
	 }
      }

      prog_file=open_pgm();
      if (prog_file==-1) {
	 error_reading_pgm(0);
	 return(0);
      }	

      par_loc = 8L;			/* # of paragraphs in EXE header */
      lcheck=lseek(prog_file,par_loc,SEEK_SET);
      if (lcheck < 0) {
	 error_reading_pgm(1);
	 return(0);
      }
      ucheck = read(prog_file,(char *)&exe_hdr_len,2);
      if (ucheck < 0) {
	 error_reading_pgm(1);
	 return(0);
      }

      par_loc = (long)exe_hdr_len * 16; /* Start search after EXE header */
      lcheck=lseek(prog_file,par_loc,SEEK_SET);
      ucheck=read(prog_file,params,SEARCH_LEN);
      if (ucheck<0 || lcheck<0) {
	 error_reading_pgm(1);
	 return(0);
      }
      for (param_scan = params;
	    (param_scan < params + SEARCH_LEN - 2) &&
	    (valid_signature((PAR_T *)param_scan) == 0);
	 param_scan++) {
      }
      par_loc += (long)(param_scan-params);
      switch (disk_version = valid_signature((PAR_T *)param_scan)) {
	 case VER_INVALID:		/* invalid */
	    sprintf(buf,"File %s: Can't set old version.",prog_name);
	    show_error(buf,10,-1,1);
	    close(prog_file);
	    return(0); 
	 case VER_NEW:		/* new - ok */
	    memcpy((char *)ptr,param_scan,sizeof(PAR_T));
      }

      if (!setting_mem) {
	 which_printer = printer_type(ptr->printer_flag);
      }
      if (setting_mem) {
	 pgm_printer = printer_type(ptr->printer_flag);
	 if (characteristics[pgm_printer]->signature != 
	    characteristics[which_printer]->signature) {
	    diff_printer = 1;	/* memory and program printers differ */
	 }
      } else {	/* Check selected printer against that of program */
	 if (characteristics[which_printer]->signature != ptr->printer_flag) {
	    diff_printer = 2;	/* selected and program printers differ */
	 }
      }
      if (diff_printer) {
	 strcpy(buf,diff_msg[diff_printer-1]);
	 strcat(buf,"/Program Printers Differ");
	 show_error(buf,10,-1,1);
	 close(prog_file);
	 return(0);
      }
      return(1);
   }

int put_program(ptr)
PAR_T *ptr;
{
    lcheck=lseek(prog_file,par_loc,SEEK_SET);
    if (lcheck==-1L)
    {   error_reading_pgm(2);	/* Error writing */
        return(0);
    }
    /* Save (new-style, i.e., EGAD 2.35 or later) parameter block: */
    ucheck=write(prog_file,(char *)ptr,sizeof(PAR_T));
    if (ucheck==-1)
    {   error_reading_pgm(2);	/* Error writing */
        return(0);
    }
    ucheck=close(prog_file);
    if (ucheck==-1)
    {   error_reading_pgm(3);	/* Error closing */
        return(0);
    }
    return(1);
}

/*	get_memory() returns the printer type (0..NPRINTERS-1) or
	-1 if there is no EGAD in memory
*/

int get_memory(ptr)
PAR_T *ptr;
{   int i;
    char *our_copy;
    char far *system_copy;
    PAR_T from_mem;

    inregs.h.ah = 0x35; 
    inregs.h.al = 0x05; 
    int86x(0x21,&inregs,&outregs,&segregs);
    FP_SEG(prtsc_vec)=segregs.es;
    FP_OFF(prtsc_vec)=0;
#if 0
    printf("\nInterrupt 5 vector %lX %lX\n",prtsc_vec,par);
    printf("Vec   %X %X %X\n",prtsc_vec[0],prtsc_vec[1],prtsc_vec[2]);
#endif
    for (i=0,system_copy=prtsc_vec,our_copy=(char *)&from_mem;
         i<sizeof(PAR_T); i++)
      { *(our_copy++) = *(system_copy++);
      }
    switch (mem_version = valid_signature(&from_mem))
      { case VER_INVALID:		/* invalid */
	  return(-1);
        default:	/* NEW - copy verbatim */
	  memcpy((char *)ptr,(char *)&from_mem,sizeof(PAR_T));
	  break;
      }
    which_printer = printer_type(ptr->printer_flag);

    return(which_printer);
}

int put_memory(ptr)
PAR_T *ptr;
   {
      int i;
      char *our_copy;
      char far *system_copy;
      PAR_T temp;

      /* Save as new-style parameter block */
      memcpy(&temp,ptr,sizeof(PAR_T));
      /* Determine number of bytes to copy. This is: */
      /* 3+2+4+16+16 = 41 int's [82 bytes] for prologue & color table */
      /*	+ 'signature' bytes of data */
      i = ptr->signature + 82;
      for (our_copy=(char *)&temp, system_copy=prtsc_vec; i>0; i--) {
	 *(system_copy++) = *(our_copy++);
      }
      return(1);
   }

void SaveSettings()
   {
      /* Save parameters to memory/disk file */
      if (setting_mem)  mem_chg=put_memory (working_parms);
      if (setting_disk) dsk_chg=put_program(working_parms);
   }

/* MAIN PROGRAM */

char *Save_Menu[]={NULL,
	"~Make More Changes",
	"~Print Color Chart",
	"Sho~w Sample Screens",
	"~Quit; Discard Changes",
	};

main (int argc,char *argv[])
   {
      char buf[80];
      static char andstr[]=" and ";
      static char *mem_not_avail = "Only available if setting Disk";
      int make_changes=1;

      working_parms = malloc(sizeof(PAR_T));
      mem_chg=dsk_chg=0;
      setting_mem=setting_disk=0;

      strcpy(buf,"Program to set ");
      strcat(buf,egad_name);
      strcat(buf," parameters");

      if (argc > 1) {
	 if (argv[1][0] == '/' || argv[1][0] == '-') {	/* Switch */
	    switch (toupper(argv[1][1])) {		/* /M = Set memory */
	       case 'M':
		  setting_mem = 1;
		  break;
#if 0
	       case 'H':
	       case '?':
#endif
	       default :
		  printf("SETEGAD [<program_name> | /M]\r\n");
		  printf("  %s\r\n",buf);
		  printf("  /H or /?   Prints this message\r\n");
		  printf("  /M         Sets memory image parameters\r\n");
		  exit(0);
	    }
	 } else {
	    setting_disk = 1;			/* not switch; assume File */
	 }
      }

      init_keys();
      set_rows(25);		/* Force alphanumeric mode */
      cursize(1);			/* Set underline cursor */
      cls();
      box(0,10,4,70,2);
      center_text(buf,1);
      center_text(VERSION,2);
      center_text("LS Software,  8139 E. Mawson,  Mesa, AZ 85207",3);

      if (setting_disk) {
	 in_mem = -1;
	 setting_mem = 0;
	 if (get_program(working_parms,argv[1]) == 0) {
	    exit_keys();
	    exit(1);
	 }
      } else {
	 in_mem = get_memory(working_parms); /* See if the driver is in memory */
	 /* If the driver isn't in memory, we can't set a memory copy. */
	 if (in_mem < 0) {
	    setting_mem = 0;
	 }

	 while (!setting_mem && !setting_disk) {
	    which_printer = in_mem;		/* Select printer from memory */

	    if ((selection=get_selection(in_mem)) <= 0) {
	       exit_keys();
	       exit(0);	/* ESC or Exit pressed -- quit. */
	    }

	    setting_disk = selection & 1;	/* Options 1 & 3 */
	    setting_mem = selection & 2;	/* Options 2 & 3 */

	    if (setting_disk) {
	       setting_disk = get_program(working_parms,NULL);
	       if (setting_disk ==0) setting_mem=0;
	       /* If user selected "Set Disk," then get a filename & try to
		* open it. Also disallow Set Memory if open fails.
		*/
	    }
	 }

	 if (setting_mem && setting_disk) {
	    in_mem = get_memory(working_parms);
	 }

	 /* If we set...	we get parameters from...
	  *	Disk only	Disk
	  *	Disk & Memory	Memory
	  *	Memory only	Memory
	  */

	 if (which_printer <0) {		/* Went to SelectPrinter menu but */
	    which_printer = in_mem;		/* escaped out -- get printer from memory */
	 }
      }

      cls();
      while (make_changes) {
	 sel_operation(working_parms,Mem_or_Disk[setting_mem?1:0]);
	 /* "Disk" if 0, "Memory" if 1 */
	 strcpy(buf,"~Save changes to ");
	 if (setting_mem)		  strcat(buf,Mem_or_Disk[1]);
	 if (setting_disk && setting_mem) strcat(buf,andstr);
	 if (setting_disk)		  strcat(buf,Mem_or_Disk[0]);
	 Save_Menu[0]=buf;
	 switch(menu("Save Menu",Save_Menu,5,10,42)) {
	    case 4:	/* Discard Changes */
	       make_changes = setting_mem = setting_disk = 0;
	       break;
	    case 3:
	       iDoTestPatterns();
	       break;
	    case 2:
	       if (setting_disk) {
		  if (sample(working_parms->printer_flag) != 0) {
		     show_error("VGA/EGA display required",12,43,1);
		  }
	       } else {
		  show_error(mem_not_avail,10,43,0);
	       }
	       break;
	    case 0:	/* Save changes */
	       make_changes=0;
	       break;
	    case -1:	/* Esc pressed */
	    case 1:	/* Make more changes -- go 'round again */
	       break;
	 }
      }

      cls();
      SaveSettings();

      setcur(2,0);
      wstr("Changes ");
      if (!mem_chg && !dsk_chg) {
	 wstr("Discarded");
      } else {
	 wstr("have been made to ");
	 if (mem_chg==1) {
	    wstr(Mem_or_Disk[1]);
	    if (dsk_chg==1) wstr(andstr);
	 }
	 if (dsk_chg==1) {
	    wstr(Mem_or_Disk[0]);
	    wstr(" (");
	    wstr(prog_name);
	    wchar(')');
	 }
      }
      wstr(".\r\n");
      erase_instruction();

      exit_keys();
      exit(0);
   }
